{ config, inputs, pkgs, lib, ... }:

with pkgs;

let
  nvidia-offload = writeShellScriptBin "nvidia-offload" ''
    #!/bin/bash
    export __NV_PRIME_RENDER_OFFLOAD=1
    export __NV_PRIME_RENDER_OFFLOAD_PROVIDER=NVIDIA-G0
    export __GLX_VENDOR_LIBRARY_NAME=nvidia
    export __VK_LAYER_NV_optimus=NVIDIA_only
    exec "$@"
  '';

in {
  imports = [
    ./hardware.nix
    ./hardware-configuration.nix # Include the results of the hardware scan.
    ./vfio.nix
    ../common/users/m3tam3re
    ../common/base
    ./services
  ];
  
  specialisation = {
    "EX-Display".configuration = {
      system.nixos.tags = [ "Externer-Monitor" ];
      services.xserver.videoDrivers = [ "nvidia" ];
      hardware.nvidia.prime.offload.enable = lib.mkForce false;
      hardware.nvidia.powerManagement.finegrained = lib.mkForce false;
    };
  };
  specialisation = {
    "DUAL-Display".configuration = {
      system.nixos.tags = [ "Dual-Monitor" ];
      services.xserver.videoDrivers = [ "nvidia" ];
      hardware.nvidia.prime.offload.enable = lib.mkForce false;
      hardware.nvidia.prime.sync.enable = lib.mkForce true;
      hardware.nvidia.powerManagement.finegrained = lib.mkForce false;
    };
  };
  specialisation = {
    "VFIO".configuration = {
      system.nixos.tags = [ "GPU-passthrough" ];
      vfio.enable = true;
    };
  };
  # Bootloader.
  # boot.loader.systemd-boot.enable = true;
  boot.loader.efi.canTouchEfiVariables = true;
  boot.loader.efi.efiSysMountPoint = "/boot/efi";
  boot.loader.grub.enable = true;
  boot.loader.grub.efiSupport = true;
  boot.loader.grub.device = "nodev";
  boot.loader.grub.useOSProber = true;

  hardware.tuxedo-keyboard.enable = true;
  boot.extraModulePackages = with config.boot.kernelPackages; [ v4l2loopback ];
  boot.kernelModules = [ "v4l2loopback" ];

  boot.kernelParams = [
    "tuxedo_keyboard.mode=0" # https://github.com/tuxedocomputers/tuxedo-keyboard#kernelparam
    "tuxedo_keyboard.brightness=255"
    "tuxedo_keyboard.color_left=0xff0a0a"
  ];
  # Setup keyfile
  boot.initrd.secrets = { "/crypto_keyfile.bin" = null; };
  boot.extraModprobeConfig = ''
    options kvm_intel nested=1
    options kvm_intel emulate_invalid_guest_state=0
    options kvm ignore_msrs=1
    options v4l2loopback exclusive_caps=1 max_buffers=2
  '';

  boot.initrd.luks.devices."luks-a7b1ba69-0951-4347-886e-4c0c24c2b871".keyFile =
    "/crypto_keyfile.bin";
  networking.hostName = "m3-nix"; # Define your hostname.
  # networking.wireless.enable = true;  # Enables wireless support via wpa_supplicant.
  networking.firewall.extraCommands =
    "iptables -t raw -A OUTPUT -p udp -m udp --dport 137 -j CT --helper netbios-ns";
  # Configure network proxy if necessary
  # networking.proxy.default = "http://user:password@proxy:port/";
  # networking.proxy.noProxy = "127.0.0.1,localhost,internal.domain";

  # Enable networking
  networking.networkmanager.enable = true;

  services.avahi = {
    enable = true;
    nssmdns = true;
    publish = {
      addresses = true;
      workstation = true;
      userServices = true;
    };
  };

  # xdg.portal = {
  #   enable = true;
  #   wlr.enable = true;
  #   extraPortals = [ pkgs.xdg-desktop-portal-gtk pkgs.xdg-desktop-portal-wlr ];
  # };
  programs.hyprland = {
    enable = true;
    nvidiaPatches = true;
  };
  programs.steam = {
    enable = true;
    remotePlay.openFirewall = true;
    dedicatedServer.openFirewall = true;
  };
  programs.fish.enable = true;
  programs.thunar = {
    enable = true;
    plugins = with pkgs.xfce; [ thunar-archive-plugin thunar-volman ];
  };
  age = {
    secrets = {
      tailscale-key.file = ../../secrets/tailscale-key.age;
      wg-key.file = ../../secrets/wg-key.age;
    };
    identityPaths = [ "/root/.ssh/lkk-nix-1" ];
  };

  time.timeZone = "Europe/Berlin";
  i18n.defaultLocale = "de_DE.utf8";
  console.keyMap = "de";

  environment.systemPackages = [ nvidia-offload neovim wally-cli ];
  nix.extraOptions = ''
    experimental-features = nix-command
  '';
  nix = {
    settings = { experimental-features = "nix-command flakes"; };
    gc = {
      automatic = true;
      options = "--delete-older-than 30d";
    };
    optimise.automatic = true;
  };
  systemd.extraConfig = ''
    DefaultTimeoutStopSec=10s
  '';
  nixpkgs.config.allowUnfree = true;

  # This value determines the NixOS release from which the default
  # settings for stateful data, like file locations and database versions
  # on your system were taken. It‘s perfectly fine and recommended to leavecatenate(variables, "bootdev", bootdev)
  # this value at the release version of the first install of this system.
  # Before changing this value read the documentation for this option
  # (e.g. man configuration.nix or on https://nixos.org/nixos/options.html).
  system.stateVersion = "23.11"; # Did you read the comment?

}
